function meu:define_meu_infos( )
	return	{ author = "Mâa",
				name_long = "Geometry with UV",
				tags = { "3D", "core", "coregraphic", "geometry", "surface", "Draw", "VJ", "unfinished" },
				help = "Display a 3D procedural object (Sphere, Torus, Cone, Teapot ...)"
			}
end

function meu:define_ui()
	local ref	 = self.ref
	local ui 	 = self.ui
	local bu

	local ix, iy = 1,1
    local SX, SY = 4/3, 1
	local DY	 = .2

	local mapping_ref = aaa.obj.get_branch_by_class( self:get_layer(1),	"mapping" )

	self:add_rendering()
	self:add_camera()

	bu = self:add_trig_method( {ix+1,iy,		   2,SY},	"Reset",			self,		"reset"		)
    	bu:set_text_visible( true )
	iy = iy + SY + DY

	bu = self:add_selector(    {ix,iy,     		   8,SY},	"Geometry"   )
		bu:set_item_text( 1, "Sphere", "Torus", "Cone", "Grid", "Teapot" )
		bu:set_target_lua( self , "selec_geometry" )
		bu:set_text_draw( false )
	iy = iy + SY + DY

	bu = self:add_selector(    {ix,iy,     		   4,SY},	"AXE"  	     )
		bu:set_item_text( 1, "X", "Y", "Z" )
		bu:set_target_lua( self , "selec_axe" )
		bu:set_text_draw( false )
		self.selec_axe = 3
	iy = iy + SY
	
	--todo at some point try to use MEU:add_sliders_suvwf() (search MEU:add_sliders_sxyzf() for examples)
	bu = self:add_slider( 	   {ix,iy,		       2,SY},	"Size",				 self,		"size",					1,		0,4		)
		bu:add_values_def( .1,.3,.5,.8,1,1.5 )
		bu:set_color_back( "factor" )
	bu = self:add_slider( 	   {ix+2,iy,           2,SY},	"Size U",			 self,		"size_u",				1,		0,4		)
		bu:set_color_back( "u" )
		bu:add_values_def( 1,2,3,4)
	bu = self:add_slider(      {ix+4,iy,           2,SY},	"Size V",			 self,		"size_v",				1,		0,4		)
		bu:set_color_back( "v" )
		bu:add_values_def( 1,2,3,4)
	bu = self:add_slider(      {ix+6,iy, 	       2,SY},	"Size W",			 self,		"size_w",				1,		0,4		)
		bu:set_color_back( "w" )
		bu:add_values_def( .1,.3,.5,.8,1,1.5 )	
	iy = iy + SY

	bu = self:add_slider(		{ix,iy,			   8,SY},	"Torus_Cone Radius", self,		"torus_cone_radius",	0,	   -1,1		)
	iy = iy + SY + DY

	bu = self:add_slider(	    {ix,iy,		 	2*SX,SY},	"Nb U",				 self,		"nb_u",				   64,	    0,128	)
		bu:set_color_back( "u" )
		bu:set_value_type_integer( true )
		bu:add_values_def( 10,20,30,50,70,80,90 )
	bu = self:add_slider( 	   {ix+2*SX,iy, 	2*SX,SY},	"Center U",			 self,		"center_u",				0,		-1,1	)
		bu:set_color_back( "u" )
	bu = self:add_slider( 	   {ix+4*SX,iy, 	2*SX,SY},	"Range U",			 self,		"range_u",				1,		-1,1	)
		bu:set_color_back( "u" )	
	iy = iy + SY

	bu = self:add_slider( 	   {ix,iy,			2*SX,SY},	"Nb V", 			 self,		"nb_v",				   64,		0,128	)
		bu:set_color_back( "v" )
		bu:set_value_type_integer( true )
		bu:add_values_def( 10,20,30,50,70,80,90 )
	bu = self:add_slider( 	   {ix+2*SX,iy, 	2*SX,SY},	"Center V",			 self,		"center_v",				0,		-1,1	)
		bu:set_color_back( "v" )
	bu = self:add_slider( 	   {ix+4*SX,iy, 	2*SX,SY},	"Range V",			 self,		"range_v",				1,		-1,1	)
		bu:set_color_back( "v" )
	iy = iy + SY + DY

	bu = self:add_button(      {ix,iy,            4,SY},	"Implicit Texture",  self,		"b_tex_implicit",    true				)
		bu:set_target_param( param.get_ref( mapping_ref,	"tex_implicit"	) )
	iy = iy + SY

	bu = self:add_slider(      {ix,iy,   	  2*SX,SY},		"Factor",            self,		"factor",				1,      	0,5	)
		bu:set_target_param( param.get_ref( mapping_ref,	"tex_factor"	) )
		bu:set_color_back( "factor" )
	bu = self:add_slider(      {ix+2*SX,iy,    2*SX,SY},	"Factor u",			 self,		"factor_u",				1,      	0,5 )
		bu:set_target_param( param.get_ref( mapping_ref,	"tex_u"			) )
		bu:set_color_back( "u" )
	bu = self:add_slider(      {ix+4*SX,iy,    2*SX,SY},	"Factor v",			 self,		"factor_v",				1,      	0,5 )
		bu:set_target_param( param.get_ref( mapping_ref,	"tex_v"			) )
		bu:set_color_back( "v" )
	iy = iy + SY

	bu = self:add_slider( 	   {ix+2*SX,iy,	   2*SX,SY},	"U Origin", 		 self,		"u_ori",				0,		   -5,5 )
		bu:set_target_param( param.get_ref(  mapping_ref,	"tex_u_ori"		) )
		bu:set_color_back( "u" )
	bu = self:add_slider( 	   {ix+4*SX,iy,	   2*SX,SY},	"V Origin", 		 self,		"v_ori",				0,		   -5,5 )
		bu:set_target_param( param.get_ref(  mapping_ref,	"tex_v_ori"		) )
		bu:set_color_back( "v" )
	iy = iy + SY + DY

	
	ix, iy = 9, 2 + DY

	self:add_transfo(		   {ix,iy,		     8,3*SY}, 1 )
	iy = iy + 3 + DY

	
	iy = iy + 3*SY + DY
	self:add_bu_texture_target_unit( {ix,iy,     8,6*SY}    )
	iy = iy + 6*SY + DY
end

function meu:init()
	
end

function meu:reset()
	self.size_u, 	 self.size_v, 	self.size_w	= 1, 1, 1
	self.u_ori,		 self.v_ori,	self.w_ori	= 0, 0, 0
	self.size,		 self.factor				= 1, 1
	self.range_u, 	 self.range_v 				= 1, 1
	self.center_u, 	 self.center_v 				= 0, 0
	self.nb_u, 		 self.nb_v					= 50, 50
	self.factor_u,	 self.factor_v 				= 1, 1
	self.torus_cone_radius						= 0
end

function meu:update()
	local ref = self.ref

	local layer				=	self:get_layer( 1 )
	local pgr				=	param.get_ref
	local gbbc  			=	aaa.obj.get_branch_by_class

		ref.model			=	gbbc( layer, "model" 		   )
		ref.axe 			=	 pgr( ref.model, "axe_object"  ) 
		ref.factor			=	 pgr( ref.model, "size_factor" )
		ref.size_u			=	 pgr( ref.model, "size_u" 	   )
		ref.size_v			=	 pgr( ref.model, "size_v" 	   )
		ref.size_w      	=    pgr( ref.model, "size_axe"    )

	local bdd_ref 			= 	 pgr( layer , "bdd" 		   )
	local bdd_torus_ref 	= 	gbbc( layer , "bdd_torus" 	   )
	local bdd_sphere_ref 	= 	gbbc( layer , "bdd_sphere" 	   )
	local bdd_cone_ref 		= 	gbbc( layer , "bdd_cone" 	   )
	local bdd_grid_ref 		= 	gbbc( layer , "bdd_grid" 	   )
	local bdd_teapot_ref 	= 	gbbc( layer , "bdd_torus" 	   ) 				

	local bdd_names 		= 	{ "sphere" , "torus" , "cone" , "grid" , "teapot" }
	local axe_names 		= 	{ "X" , "Y" , "Z" }
	local bdd_general 		= 	{ bdd_sphere_ref, bdd_torus_ref , bdd_cone_ref , bdd_grid_ref , bdd_teapot_ref	}	

		ref.range_u			=	pgr( bdd_general[self.selec_geometry], "u_range"  )
		ref.range_v			=	pgr( bdd_general[self.selec_geometry], "v_range"  )
		ref.nb_u			=	pgr( bdd_general[self.selec_geometry], "nb_u" 	  )
		ref.nb_v			=	pgr( bdd_general[self.selec_geometry], "nb_v" 	  )
		ref.center_u		=	pgr( bdd_general[self.selec_geometry], "u_center" )
		ref.center_v		=	pgr( bdd_general[self.selec_geometry], "v_center" )
		ref.radius_int		=	pgr( bdd_torus_ref, "radius_int" 				  )
		ref.radius_top		=	pgr( bdd_cone_ref,  "radius_top" 				  )


		param.set( 		bdd_ref 	   , 		bdd_names[self.selec_geometry] )
		param.set( 		ref.axe 	   , 		axe_names[self.selec_axe] 	   )
		param.set( 		ref.factor 	   , 		self.size			   		   )
		param.set( 		ref.size_u 	   , 		self.size_u	   			       )
		param.set( 		ref.size_v 	   , 		self.size_v		               )
		param.set( 		ref.size_w 	   , 		self.size_w			   		   )
		param.set( 		ref.range_u    ,  		self.range_u		   		   )
		param.set( 		ref.range_v    , 		self.range_v		   		   )
		param.set( 		ref.nb_u 	   ,		self.nb_u		   			   )
		param.set( 		ref.nb_v 	   ,		self.nb_v		  			   )
		param.set( 		ref.center_u   , 		self.center_u		  		   )
		param.set( 		ref.center_v   , 		self.center_v		 		   )
		param.set( 		ref.radius_int ,	 	self.torus_cone_radius 		   )
		param.set( 		ref.radius_top , 		self.torus_cone_radius		   )
end


