function meu:define_meu_infos()
    return { author = "Abdalight", date = "2024",
               tags = { "2d", "3d", "Art", "draw", "procedural", "unfinished", "vj" },
               help = "An animated falling code effect inspired by the Matrix movie, with more columns for a denser effect!"
    }
end

function meu:define_ui()
    local ix, iy = 1, 2
    local SY = 1
    local bu
    local ui = self.ui

    -- Ajouter le rendu et la caméra
    self:add_rendering()
    self:add_camera()

    ui.color = self:add_rgba(   {ix,iy,         8,SY},      "Color",        false )
    iy = iy + SY

    -- Ajout des sliders pour la vitesse, la densité et le nombre de colonnes
    bu = self:add_slider(       {ix,iy,         4,SY},      "Fall Speed",   self,   "fall_speed",       0.1,    0.01,1.0    )
    iy = iy + SY

    bu = self:add_slider(       {ix,iy,         4,SY},      "Density",      self,   "density",           20,    1,1000      )
        bu:set_value_type_integer( true )
    iy = iy + SY

    bu = self:add_slider(       {ix,iy,         4,SY},      "Columns",      self,   "frame_width",      150,    50,300      )
        bu:set_value_type_integer( true )
    iy = iy + SY

    bu = self:add_trig_method(  {ix,iy,         4,SY},      "RESTART",      self,   "restart_command"                       )
    iy = iy + SY

    bu = self:add_slider(       {ix,iy,         4,SY},      "TESTO",        self,   "test",               1,     0,10       )
    iy = iy + SY
end

function meu:init()
    -- Initialisation des paramètres globaux
    self.frame_width = 50 -- Nombre de colonnes par défaut
    self.frame_height = 30 -- Hauteur de l'écran
    self.characters = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
    self.c_map = nil  -- Contiendra des chaînes pour chaque colonne
    self.rain = nil   -- Contiendra les positions des chaînes
    self.fall_speed = 0.1
    self.density = 20  -- Longueur par défaut des chaînes verticales

    self.x_spacing = 0.9 -- Utiliser la largeur d'un caractère

    -- Initialisation complète
    self:reset_animation()
end


function meu:reset_animation()
    -- Réinitialiser les données de simulation
    aaa.print_debug("Réinitialisation de l'animation...")
    self.c_map = {}
    self.rain = {}
    math.randomseed(420)

    -- Génération des chaînes verticales (une par colonne)
    for i = 1, self.frame_width do
        local column = ""
        for j = 1, self.density do
            column = column .. string.sub(self.characters, math.random(1, #self.characters), 1)
        end
        self.c_map[i] = column
        table.insert(self.rain, {pos = math.random(1, self.frame_height), speed = math.random(1, 5)})
    end
end



function meu:restart_command()
    self:reset_animation()
end




function meu:draw()
    self:draw_layers_begin()
        self:matrix_draw()
    self:draw_layers_end()
end





function meu:update()
    -- Validation des données
    if type(self.rain) ~= "table" then
        aaa.print_error("self.rain n'est pas une table valide ! Réinitialisation nécessaire.")
        self:reset_animation()
        return
    end

    -- Mise à jour des positions des chaînes
    for i, drop in ipairs(self.rain) do
        if drop.pos and drop.speed then
            drop.pos = drop.pos - self.fall_speed * drop.speed -- Déplacer vers le bas
            if drop.pos < 1 then
                drop.pos = self.frame_height -- Réinitialisation en bas
                self.c_map[i] = string.sub(self.characters, math.random(1, #self.characters), math.random(1, #self.characters)) -- Changer de caractère
            end
        else
            aaa.print_error("Une goutte est invalide :", drop)
        end
    end
end

function meu:matrix_draw()
    self.x_spacing = self.test

    local r, g, b, a = self.ui.color:get_rgba()

    -- Validation des données avant le rendu
    if type( self.c_map ) ~= "table" then
        aaa.print_error("self.c_map n'est pas initialisé ! Réinitialisation nécessaire.")
        self:reset_animation()
        return
    end

    if type( self.rain ) ~= "table" then
        aaa.print_error("self.rain n'est pas initialisé ! Réinitialisation nécessaire.")
        self:reset_animation()
        return
    end

    -- Centrer le mur sur (0,0)
    local start_x = - self.frame_width * self.x_spacing * 0.5
    local start_y = - self.frame_height * 0.5
    local z_decal = 0   --5 * ( math.sin( aaa.time.t ) )
    aaa.show(z_decal)
    -- Dessiner les chaînes verticales
    for x = 1, self.frame_width do
        local column = self.c_map[x]
        local drop = self.rain[x]

        if drop then
            -- Calculer la position de la colonne
            local draw_x = start_x + ( x - 1 ) * self.x_spacing
            local draw_y = start_y + drop.pos

            -- Afficher chaque caractère de la chaîne
            for i = 1, #column do
                local char = string.sub( column, i, i )
                local char_y = draw_y + i
                if char_y >= start_y and char_y <= self.frame_height + start_y then
                    if i == 1 then
                        gol.color(1, 1, 1, 1) -- Blanc pour la tête
                    else
                        local brightness = math.max(0.2, 1.0 - (i / self.density))
                        --gol.color(0, brightness, 0, 1) -- Vert pour le reste
                        gol.color(r, brightness*g, b, a)
                    end

                    -- Dessiner le caractère
                    aaa.draw_str_maa_xyz(char, draw_x, char_y, z_decal)
                end
            end
        else
            aaa.print_error("Aucune goutte trouvée pour la colonne " .. x)
        end
    end
end