function meu:define_meu_infos( )
	return { author = "Abdalight", date="2024",
			tags = { "3D", "Art", "Draw", "Generator", "Procedural", "VJ" },
			help="Ploting a strange attractor with Lorentz setup" }
end

function meu:define_ui()
	local ref   = self.ref
	local ui    = self.ui
	local bu
	local sha   = self:get_shading()
	local ix,iy = 1,1
	local SY,DY = 1,.2

	self:add_shading_ui(			{ix,iy,		8,SY} )
	self:add_slider(				{ix+4,iy,	4,SY},	"Nb",			ref.vertex_nb,nil,		2048,	1,1024*1024*8)
		:set_value_type_integer(true)
	iy = iy + SY + DY

	local y_begin = iy

------------------------------
	self:set_tab_key( "Main" )
	ix, iy = 1,y_begin

	self:add_button(				{ix,iy,		1,SY},	"Time Active",	self,	"b_time",	false	):set_text_visible(false)
	self:add_slider(				{ix+1,iy,	5,SY},	"Time",			self,	"time",	0,		0,100 )	
	self:add_trig_method(			{ix+6,iy,	2,SY},	"Restart",		self,	"restart"	)
	iy = iy + SY + DY
	self:add_transfo(				{ix,iy,	 nil,2.4},  1 )
	iy = iy + 2.4 * SY + DY

	self:add_bu_texture_target_unit_nb( {ix,iy,	8,5},	"Tex_", 3,1 )
------------------------------
	--self:add_bu_texture_target_unit_nb( 		{ix,5.8,	16,8},	"Tex_", 2,2 )	

	ix,iy = 9,y_begin
	self:add_camera()

	ui.bu_viz = self:add_rgbf(		{ix,iy,		8,SY}	)
	iy = iy + SY + DY

	self:add_slider({ix, iy, 3, SY}, "Mix Begin 1", self, "u_mix_begin_1", 0.0, 0.0, 1.0)
	self:add_slider({ix+3, iy, 3, SY}, "Mix End 1", self, "u_mix_end_1", 0.5, 0.0, 1.0)
	iy = iy + SY + DY
	self:add_slider({ix, iy, 3, SY}, "Mix Begin 2", self, "u_mix_begin_2", 0.5, 0.0, 1.0)
	self:add_slider({ix+3, iy, 3, SY}, "Mix End 2", self, "u_mix_end_2", 1.0, 0.0, 1.0)
	
	self:add_rendering()

------------------------------
	local sha = self:get_shading()
	self:add_shading_sliders_tab(	sha,	{"Vert","Frag"},	y_begin,SY	)

	ui.bu_time_vert 	 = self:get_bu_by_key( "vertex_float_1" )
	ui.bu_vertex_nb_over = self:get_bu_by_key( "vertex_float_2" )
	ui.bu_time_geom 	 = self:get_bu_by_key( "geometry_float_1" )
	ui.bu_time_frag 	 = self:get_bu_by_key( "fragment_float_1" )
end

function meu:init()
	local ref = self.ref

	self.layer_shading = 2
	local sha = self:add_shading( self.layer_shading )

	--ref.bdd			=	self:get_layer_bdd( 3 )
	--ref.vertex_nb	=	param.get_ref( ref.bdd, "vertex_nb" )
	--self.u_mix_begin_1 = 0.0
	--self.u_mix_end_1   = 0.5
	--self.u_mix_begin_2 = 0.5
	--self.u_mix_end_2   = 1.0
end

function meu:restart()
	self.time = 0
end

function meu:update()
	local ref = self.ref
	local ui = self.ui
	local bu

	local t = self.time
	if self.b_time then
		t = t + aaa.time.dt
		self.time = t
	end

	if t > 15  then
		self.time = 0
	end

	ui.bu_time_vert:set_value( t )
	bu = ui.bu_time_geom
	if bu then bu:set_value( t ) end
	ui.bu_time_frag:set_value( t )

	--local nb = param.get( ref.vertex_nb )
	--ui.bu_vertex_nb_over:set_value( 1./nb )

	local sha = self:get_shading()
	--sha:set_frag_float_1_4(self.u_mix_begin_1, self.u_mix_end_1, self.u_mix_begin_2, self.u_mix_end_2)
end

function meu:draw()
	self:draw_layers_begin()
		self:draw_layer( 1 ) -- attribut
		self:draw_layer( 2 ) -- shader
		--self:draw_layer( 3 ) -- instruction
		self:draw_slot()
	self:draw_layers_end()
end


function meu:draw_slot()
	local x1,y1,z1 = -2.5, -2, 0
	local x2,y2,z2 =  2.5,  2, 0

	local function deform_tube_y(x, y, z)
		x = x * inverse_deformation_radius - pi_half
		local rho = z + deformation_radius
		return rho * math.cos(x), y, - rho * math.sin(x) - sphere_offset
	end

	aaa.draw_rect_uv(x1,y1, x2,y2)
end