function meu:define_meu_infos( )
	return { author = "Abdalight", date = "2024",
               tags = { "2d", "3D", "Art", "Draw", "Generator", "Geometry", "Procedural" },
               help = "Meu for control the Meu Curve3d" }
end


function meu:define_ui()
	local ref = self.ref
	local ui = self.ui
    local ix,iy = 1,1
    local SY,DY = 1,.2
    local bu
    local SX = 8/3



    self:add_rendering()    
	self:add_camera()

    self:set_tab_key_def()
    iy = iy + SY + DY
    
    bu = self:add_slider_xy(   {ix,iy,        16,9},     "Shift to not draw",   self,       "pos_x", "pos_y",  0,0,     -1,1,   -1,1 )
        bu:set_text_visible( false )
		ui.bu_pos = bu
    iy = iy + DY + 9 * SY


    self:begin_bu_group("Curve")
    bu = self:add_trig_method( {ix,iy,       SX,SY},     "Prev",                self,       "curve_prev"        )
    bu = self:add_slider(      {ix+SX,iy,    SX,SY},     "Curve",               self,       "s_curve", 1,  0,20 )
        bu:set_value_type_integer( true )
        bu:set_target_param( self.meu_dst.ref.curve_id )
        bu:set_target_lua(   self,      "s_curve"      )
    bu = self:add_trig_method( {ix+2*SX,iy,  SX,SY},     "Next",                self,       "curve_next"        )
    iy = iy + SY + DY

    bu = self:add_trig_method( {ix,iy,        4,SY},     "Delete",              self,       "delete_curve"      )
    bu = self:add_trig_method( {ix+4,iy,      4,SY},     "Delete All",          self,       "delete_all_curve"  )
    self:end_bu_group()

    ix, iy = 9, iy - SY - DY

    self:begin_bu_group("Drawing Zone")
    bu = self:add_slider(      {ix,iy,        4,SY},     "X size",              self,       "x_size",   16,     0,16    )
        bu:set_value_type_integer( true )
        bu:set_color_back( "x" )
    bu = self:add_slider(      {ix+4,iy,      4,SY},     "X Offset",            self,       "x_off",    0,      -5,5    )
        bu:set_value_type_integer( true )
        bu:set_color_back( "x" )
    iy = iy + SY + DY

    bu = self:add_slider(      {ix,iy,        4,SY},     "Y Size",              self,       "y_size",   9,       0,9    )
        bu:set_value_type_integer( true )
        bu:set_color_back( "y" )
    bu = self:add_slider(      {ix+4,iy,      4,SY},     "Y Offset",            self,       "y_off",    0,      -5,5    )
        bu:set_value_type_integer( true )
        bu:set_color_back( "y" )
    iy = iy + SY + DY

    bu = self:add_slider(      {ix,iy,        4,SY},     "Z Size",              self,       "z_size",   1,      0,9     )
        bu:set_value_type_integer( true )
        bu:set_color_back( "z" )
    bu = self:add_slider(      {ix+4,iy,      4,SY},     "Z Offset",            self,       "z_off",    0,     -5,5     )
        bu:set_value_type_integer( true )
        bu:set_color_back( "z" )
    iy = iy + SY + DY
    self:end_bu_group()    
end


function meu:init()
    local ref     = self.ref
    local bdd     = self:get_layer_bdd( 1 )
    local pgr     = param.get_ref
    self.meu_dst  = self:get_meu_by_name( "TutoCurve3d_1" )

    if bdd then
        ref.bdd   = bdd
        ref.pos_x = pgr( bdd, "emission_origin_x" )
        ref.pos_y = pgr( bdd, "emission_origin_y" )
        ref.pos_z = pgr( bdd, "emission_origin_z" )
    end
    self.pos_z = 0
    self.x_last, self.y_last, self.z_last = 0, 0, 0
end



function meu:delete_curve()
    local meu_dst = self.meu_dst
    if meu_dst and meu_dst.delete_curve then
        meu_dst:delete_curve()
        meu_dst:prev_curve()
    end
end

function meu:delete_all_curve()
    local meu_dst = self.meu_dst
    if meu_dst then
        meu_dst:clear_all()
    end
    self.pos_z = 0
end

function meu:curve_prev()
    local meu_dst = self.meu_dst
    if meu_dst and meu_dst.prev_curve then
        meu_dst:prev_curve()
    end
end

function meu:curve_next()
    local meu_dst = self.meu_dst
    if meu_dst and meu_dst.next_curve then
        meu_dst:next_curve()
    end
end

function meu:distance3D( x1,y1,z1, x2,y2,z2 )
    local dx = x2 - x1
    local dy = y2 - y1
    local dz = z2 - z1
    return math.sqrt( dx * dx + dy * dy + dz * dz )
end

function meu:update()
    local ref = self.ref
    local ui = self.ui
    local meu_dst = self.meu_dst   
    
    local x = self.pos_x * self.x_size + self.x_off
    local y = self.pos_y * self.y_size + self.y_off
    local z = self.pos_z --* self.z_size + self.z_off

    param.set( ref.pos_x, x )
    param.set( ref.pos_y, y )
    param.set( ref.pos_z, z )

    if meu_dst and meu_dst.new_curve and not aaa.keyboard.is_shift() then
        local b_contact = ui.bu_pos:is_contact()
        local b_contact_last = self.b_contact_last
        local x_last, y_last, z_last = self.x_last, self.y_last, self.z_last

        if b_contact  then
            if b_contact ~= b_contact_last then
                meu_dst:next_curve()
                meu_dst:new_curve()
                self.pos_z = self.pos_z + 0.001
            end
            if self:distance3D( x,y,z, x_last, y_last, z_last ) >= .2 then
                meu_dst:new_point_control( x,y,z )
                self.x_last, self.y_last, self.z_last = x,y,z
            end
        end
        
        self.b_contact_last = b_contact
    end

end

function meu:draw()
    self:draw_layers_begin()
        self:draw_layer( 1 )
        self:get_alpha()
        self:draw_screen()
        --self:draw_canvas()
    self:draw_layers_end()
end

function meu:draw_screen()
    local alpha = self:get_alpha()
    gol.color4( 1, 1, 1, alpha )                                              --16:9 white screen defaults settings
    gol.draw_line_strip_3d(	-16,-9,self.pos_z,  16,-9,self.pos_z,   16,9,self.pos_z,    -16,9,self.pos_z,   -16,-9,self.pos_z )
    gol.color4( 1, 0, 0, alpha )                                              --zooming red screen selected by bu
    gol.draw_line_strip_3d(	
        -self.x_size + self.x_off, -self.y_size + self.y_off, self.z_pos,
         self.x_size + self.x_off, -self.y_size + self.y_off, self.z_pos,
         self.x_size + self.x_off,  self.y_size + self.y_off, self.z_pos,
        -self.x_size + self.x_off,  self.y_size + self.y_off, self.z_pos,
        -self.x_size + self.x_off, -self.y_size + self.y_off, self.z_pos  )
end

function meu:draw_canvas()
    local alpha = self:get_alpha()
    gol.color4( 1, 1, 1, alpha )
    gol.draw_quads_2d( -16,-9,  16,-9,  16,9,   -16,9 )
end