function meu:define_meu_infos()
	return { author = "Abdalight", date = "2025",
			   tags = { "Tutorial", "Texture" },
			   help = "basic example of how to load images from folder"
			 }
end

function meu:define_ui()
	local ref   = self.ref
	local ui    = self.ui
    local ix,iy = 1,1
    local SY,DY = 1,.2
    local bu

    bu = self:add_bu_texture_target_unit( {ix,iy,   8,6*SY,b_compact=true},       "Tex",      1,      true          )
        ui.bu_text = bu
    iy = iy + 6 * SY + DY

    bu = self:add_trig_method(            {ix,iy,                    8,SY},       "LOAD Auto 1",  self,      "load_auto_1" )
    iy = iy + SY + DY

    bu = self:add_trig_method(            {ix,iy,                    8,SY},       "LOAD Auto 2",  self,      "load_auto_2" )
    iy = iy + SY + DY

    bu = self:add_trig_method(            {ix,iy,                    8,SY},       "LOAD Manual",  self,      "load_manual" )
    iy = iy + SY + DY
end

function meu:init()
    self.loaded_images = {}
end



function meu:load_auto_1()
    local folder_path = aaa.file.do_dialog_folder( "Select Folder for Images" )

    self:load_images_auto1( folder_path )
    if self.loaded_images then
        for i, image_data in ipairs( self.loaded_images ) do
            self:print( "Image index : " .. i )
            table.print( image_data, "data" )
        end
    else
        self:print( "No images loaded." )
    end
end

function meu:load_auto_2()
    local folder_path = aaa.file.do_dialog_folder( "Select Folder for Images" )

    self:load_images_auto2( folder_path )
    if self.loaded_images then
        for i, image_data in ipairs( self.loaded_images ) do
            self:print( "Image index : " .. i )
            table.print( image_data, "data" )
        end
    else
        self:print( "No images loaded." )
    end
end

function meu:load_manual()
    local folder_path = aaa.file.do_dialog_folder( "Select Folder for Images" )

    self:load_images_manual( folder_path )
    if self.loaded_images then
        for i, image_data in ipairs( self.loaded_images ) do
            self:print( "Image index : " .. i )
            table.print( image_data, "data" )
        end
    else
        self:print( "No images loaded." )
    end
end

--method to load images from folder
--using IMGS.get_bind() and setting the bind automatically
function meu:load_images_auto1( folder_path )
    local images = {}
    local index = 1
    local files = aaa.dir.get_files( folder_path )
    --local bind

    if not files or #files == 0 then
        self:print( "Error: No files found in folder : " .. folder_path )
        return
    end

    MEDIA.set_dir( folder_path )
    for _, file_name in ipairs( files ) do
        if string.match( file_name, "%.png$" ) or string.match( file_name, "%.jpg$" ) or string.match( file_name, "%.jpeg$" ) then
            local bind = IMGS.get_bind( file_name )
            self.ui.bu_text:set_bind_2d( bind )
            if bind then
                images[index] = {
                         file = file_name,
                         bind = bind,
                                }
                index = index + 1
            else
                self:print( "Error: Failed to load image : " .. file_name )
            end
        else
            self:print( "Skipping non-image file : " .. file_name )
        end
    end
    self.loaded_images = images
    self:print( "Loaded " .. #images .. " images from folder : " .. folder_path )
end



--other method to load images from folder
--using aaa.img.read() and setting the bind automatically
function meu:load_images_auto2( folder_path )
    local images = {}
    local index = 1
    local files = aaa.dir.get_files( folder_path )
    local bind

    if not files or #files == 0 then
        self:print( "Error : No files found in folder : " .. folder_path )
        return
    end

    for _, file_name in ipairs( files ) do
        local full_path = folder_path .. "/" .. file_name

        if string.match( file_name, "%.png$" ) or string.match( file_name, "%.jpg$" ) or string.match( file_name, "%.jpeg$" ) then
            bind = IMGS.get_bind_free() -- return a free bind
            if bind then
                local success = aaa.img.read( bind, full_path, false, false, false, true )
                if success then
                    images[ index ] = {
                               file = file_name,
                               bind = bind,
                                      }
                    index = index + 1
                else
                    self:print( "Error: Failed to load image: " .. file_name )
                end
                IMGS.inc_bind_free( 1 ) -- now allocate the free bind we used
            else
                self:print( "Warning: No free binds available for " .. file_name )
            end
        else
            self:print( "Skipping non-image file: " .. file_name )
        end
    end
    self.ui.bu_text:set_bind_2d( bind )
    self.loaded_images = images
    self:print( "Loaded " .. #images .. " images from folder: " .. folder_path )
end


-- method to load images from folder using a user-specified bind
function meu:load_images_manual( folder_path )
    local files = aaa.dir.get_files( folder_path )
    local images = {}
    local index = 1
    local bind = self.ui.bu_text:get_bind_2d()

    if not files or #files == 0 then
        self:print( "Error: No files found in folder: " .. folder_path )
        return
    end

    for _, file_name in ipairs(files) do
        local full_path = folder_path .. "/" .. file_name
        if string.match(file_name, "%.png$") or string.match(file_name, "%.jpg$") or string.match(file_name, "%.jpeg$") then
            local success = aaa.img.read( bind, full_path, false, false, false, true )
            if success then
                images[index] = {
                    file = file_name,
                    bind = bind,
                }
                self:print( "Loaded image: " .. file_name .. " with bind: " .. bind )
                index = index + 1
                bind = bind + 1
            else
                self:print( "Error: Failed to load image: " .. file_name )
            end
        else
            self:print( "Skipping non-image file: " .. file_name )
        end
    end
    self.loaded_images = images
    self:print( "Manually loaded " .. #images .. " images from folder: " .. folder_path )
end
