function meu:define_meu_infos()
	return { author = "Benjamin", date = "2025",
			   tags = { "Tutorial", "Network" },
			   help = "Basic example of how to execute HTTP requests and examine responses"
			 }
end

function meu:init()
    self.text_str = "BONJOUR"
    self.response = nil
    self.request_id = 0
end

function meu:define_ui()
    local ref = self.ref
    local ui = self.ui
    self:add_text({1, 1, 12, 1}, "URL"):set_value_load_save(true)
    self:add_text({1, 2, 12, 1}, "Payload"):set_value_load_save(true)
    self:add_text({1, 3, 6, 1}, "Username"):set_value_load_save(true)
    self:add_text({7, 3, 6, 1}, "Password"):set_value_load_save(true)
    self:add_text_info({1, 4, 3, 1}, "Set Auth:")
    self:add_trig_method({4, 4, 3, 1}, "BASIC", self, "set_basic_auth")
    self:add_trig_method({7, 4, 3, 1}, "DIGEST", self, "set_digest_auth")
    self:add_trig_method({10, 4, 3, 1}, "NTLM", self, "set_ntlm_auth")
    self:add_text({1, 5, 12, 1}, "Bearer Token"):set_value_load_save(true)
    self:add_text_info({1, 6, 3, 1}, "Set OAuth:")
    self:add_trig_method({4, 6, 3, 1}, "OAUTH", self, "set_oauth")
    self:add_trig_method({1, 7, 4, 1}, "CLEAR AUTH", aaa.net_requests, "clear_auth")
    self:add_trig_method({5, 7, 4, 1}, "ENABLE AUTH", aaa.net_requests, "enable_auth")
    self:add_trig_method({9, 7, 4, 1}, "DISABLE AUTH", aaa.net_requests, "disable_auth")
    self:add_trig_method({1, 9, 2, 1}, "GET", self, "get_request")
    self:add_trig_method({3, 9, 2, 1}, "POST", self, "post_request")
    self:add_trig_method({5, 9, 2, 1}, "PUT", self, "put_request")
    self:add_trig_method({7, 9, 2, 1}, "DELETE", self, "delete_request")
    self:add_trig_method({9, 9, 2, 1}, "DOWNLOAD", self, "download_request")
    self:add_text_info({1, 10, 16, 1}, "Response")
    
end

function meu:draw()
    self:draw_layers_begin()
        --  self:print( aaa.time.t )
        self:draw_layer( 1 )
        gol.color_white( self:get_alpha() )
        aaa.draw_str( self.text_str )
    self:draw_layers_end()
end

function meu:update()
    local request_list = aaa.net_requests.get_request_list()
    for k, v in pairs(request_list) do
        if (v == 1) then
            local result = aaa.net_requests.get_request_result_if_done(k)
            if result ~= false then
                self:print("Request complete!")
                self:print(result.status_code)
                self:print(result.text)
                self:print(result.url)
                self:print(result.elapsed)
                -- Parse the text from the response (which is a JSON object) into a Lua table for ease of use
                self.response = aaa.net_requests.json_to_table(result.text)
                self:set_bu_value("Response", result.text)
                self:print_table(self.response, 0)
            else
                self:print("Error with request endpoint or payload!")
            end
            -- self.text_str = response.data.content
        end
    end
end

function meu:print_table(t, d)
    for k, v in pairs(t) do
        if type(v) == "table" then
            self:print(string.rep("  ", d)..k.." : [")
            self:print_table(v, d+1)
        else
            self:print(string.rep("  ", d)..k.." : "..v)
        end
    end
    self:print(string.rep("  ", d).."]")
end

function meu:net_request()
    local link = "https://animechan.io/api/v1/quotes/random"
    self.request_id = aaa.net_requests.request_get(link)
    self:print("Request ID: "..self.request_id)
end

function meu:get_request()
    local url = self:get_bu_value("URL")
    self.request_id = aaa.net_requests.request_get(url)
    self:print("Request ID: "..self.request_id)
end

function meu:post_request()
    local url = self:get_bu_value("URL")
    local payload = self:get_bu_value("Payload")
    if url == nil then
        self:print("No URL provided")
        return
    end
    if payload == nil then
        payload = ""
    end
    self:print("URL: "..url..", Payload: "..payload)
    self.request_id = aaa.net_requests.request_post(url, payload)
    self:print("Request ID: "..self.request_id)
end

function meu:put_request()
    local url = self:get_bu_value("URL")
    local payload = self:get_bu_value("Payload")
    if url == nil then
        self:print("No URL provided")
        return
    end
    if payload == nil then
        payload = ""
    end
    self.request_id = aaa.net_requests.request_put(url, payload)
    self:print("Request ID: "..self.request_id)
end

function meu:delete_request()
    local url = self:get_bu_value("URL")
    if url == nil then
        self:print("No URL provided")
        return
    end
    self.request_id = aaa.net_requests.request_delete(url)
    self:print("Request ID: "..self.request_id)
end

function meu:download_request()
    local url = self:get_bu_value("URL")
    local path = self:get_bu_value("Payload")
    if url == nil or url == "URL" then
        self:print("No URL provided")
        return
    end
    if path == nil or path == "Payload" then
        path = "./download"
    end
    self.request_id = aaa.net_requests.request_download(url, path)
    self:print("Request ID: "..self.request_id)
end

function meu:set_basic_auth()
    local user = self:get_bu_value("Username")
    local pw = self:get_bu_value("Password")
    if user == nil or user == "" then
        self:print("No username provided")
        return
    end
    if pw == nil or pw == "" then
        self:print("No password provided")
        return
    end
    aaa.net_requests.set_basic_auth(user, pw)
end

function meu:set_digest_auth()
    local user = self:get_bu_value("Username")
    local pw = self:get_bu_value("Password")
    if user == nil or user == "" then
        self:print("No username provided")
        return
    end
    if pw == nil or pw == "" then
        self:print("No password provided")
        return
    end
    aaa.net_requests.set_digest_auth(user, pw)
end

function meu:set_ntlm_auth()
    local user = self:get_bu_value("Username")
    local pw = self:get_bu_value("Password")
    if user == nil or user == "" then
        self:print("No username provided")
        return
    end
    if pw == nil or pw == "" then
        self:print("No password provided")
        return
    end
    aaa.net_requests.set_ntlm_auth(user, pw)
end

function meu:set_oauth()
    local token = self:get_bu_value("Bearer Token")
    if token == nil or token == "" or token == "Bearer Token" then
        self:print("No token provided")
        return
    end
    aaa.net_requests.set_oauth(token)
end

