if CLASS.DECLARE( "BIND_TEX" ) then
	BIND_TEX:set_class_status_doc(	CLASS.STATUS.CORE,
									"centralize code to deal with bank of texture used by TEXS" )
end


function BIND_TEX:create( name )
	local self = BIND_TEX:create_instance( name )
	self.__info = {}
	self.__name_to_info = {}
	self.__bind_to_info = {}
	self.__b_need_compute = true
	return self
end

function BIND_TEX:add_name_bind( group, name, bind, name_short, syno )
	table.insert( self.__info,	{ group=group, name=name, bind_flat=bind, name_short=name_short, syno=syno } )
	--aaa.box_good( elt[1].." is bind "..elt[2] )
end
function BIND_TEX:add_table( group, tab )
	for _,e in IPAIRS( tab ) do
		self:add_name_bind( group, e[1], e[2], e[3], e[4] )
	end
end
function BIND_TEX:add_pre( group, pre_or_fn_i, bind_begin, nb )
	local b_str = type(pre_or_fn_i) == "string"
	for i=1,nb do
		self:add_name_bind( group, b_str and pre_or_fn_i..i or pre_or_fn_i(i), bind_begin+i-1 )
	end
end

function BIND_TEX:compute( bind_begin, nb )
	self.__name_to_info = {}
	self.__bind_to_info = {}
	for key, elt in ipairs(self.__info) do
		self.__bind_to_info[elt.bind_flat] = elt
		local name = string.lower( elt.name )
		elt.name_lower = name
		elt.bank, elt.bind = aaa.img.make_bank_bind_2d( elt.bind_flat )
		--store under both name and name_short and synonyme
		self.__name_to_info[name] = elt

		name = elt.name_short
		if name then
			self.__name_to_info[ string.lower(name) ] = elt
		end

		name = elt.syno
		if name then
			self.__name_to_info[ string.lower(name) ] = elt
		end
	end
	self.__b_need_compute = true
end

-- function BIND_TEX:find_index_by_bank_bind( bank, bind )
-- 	local tab = self.info
-- 	local nb = #tab
-- 	for i=1,nb do
-- 		local t = tab[i]
-- 		if bind == t[4] and bank == t[3] then
-- 			return i
-- 		end
-- 	end
-- 	return nil
-- end
function BIND_TEX:get_info_by_bind( bind )
	return bind and self.__bind_to_info[ bind ]
end
function BIND_TEX:get_name_from_bind( bind )
	local info = self:get_info_by_bind( bind )
	return info and info.name or string.format( "bind_%d", bind )
end
function BIND_TEX:get_name_or_bind( bind )
	local info = self:get_info_by_bind( bind )
	return info and info.name or tostring(bind)
end
function BIND_TEX:get_group( bind )
	local info = self:get_info_by_bind( bind )
	return info and info.group or nil
end

function BIND_TEX:get_bank_bind_by_name( name )
	local elt = self.__name_to_info[ string.lower(name) ]
	if elt then	return elt.bank, elt.bind	end
end
function BIND_TEX:get_bind_by_name( name )
	name = string.lower(name)
	local elt = self.__name_to_info[name]
	if elt then
		return elt.bind_flat
	else
		self:print_error( "BIND_TEX did not find "..name )
		--table.print( self.__name_to_info, "name_to_info" )
	end
end

function BIND_TEX:get_infos_by_group( group )
	local tab = {}
	for i,e in IPAIRS(self.__info) do
		--self:print( i.." -> "..e.group.." "..e.name )
		--table.print( e, 3 )
		if e.group==group then
			table.insert( tab, e )
		end
	end
	return tab
end


--todo deal with sublevels
--todo deal with relation to this with IMG,IMGS,IMAGE...
--todo fn about bank,bind ...
--todo move to TEXS.lua file ?
-- central mecanism to acess texture
if CLASS.DECLARE( "TEXS" ) then
	TEXS:set_class_status_doc(	CLASS.STATUS.CORE,
								"bank of texture should be accessed through here" )
	--old comments 
	--we use intermediate fns to change the low one in real time
	--local function set_tex_src_from_selector( bu, balue ) MEU.set_tex_src_from_selector( bu, balue ) end

	local bind_tex = BIND_TEX:create( "TEXS" )

	--		name			bind name_short	synonym
	local pass =
	{
		{ "Albedo", 		188, "Albe",	"GBuffer"	},
		{ "Normal", 		189, "Nor",		},
		{ "Specular",		190, "Spec",	},
		{ "Emissive",		191, "Emis"		},

		{ "ZBuf",			187, "ZBuf"		},
		{ "Light",			186, 			},
		{ "Fog",			185, 			},
		{ "Dof",			184, 			},
		{ "Transparent",	183, "Transp"	},
		{ "GodRays",		182, "Rays"		},
		{ "ToneMap",		181, "Tone"		},
		{ "Fxaa", 			180,			},
		{ "Caustic", 		179,			},		--todo remove for a more generic one soon (2021 October)
		{ "FogVolume", 		178, "FogVol"	},
		{ "AddBlur", 		177, "Blur"		},
		{ "Debug", 			176, "Dbg"		},

		{ "Shadow Map 1", 	212, "Shadow1"	},	--, "Shadow_1"	},
		{ "Shadow Map 2", 	213, "Shadow2"	},	--, "Shadow_2"	},
	}
	
	local dif =
	{
		{ "Pre", 			112,			},
		{ "Viz", 			113,			},
		{ "Dif",			114,			},
		{ "DifR",			115,			},
	}
	local CAP_BEGIN = 128
	local CAP_END	= CAP_BEGIN+31
	local cap =
	{
		{ "Capture.In1", 			CAP_BEGIN+0,	"Cap.In1"	},
		{ "Capture.In2",			CAP_BEGIN+1,	"Cap.In2"	},
		{ "Capture.In3",			CAP_BEGIN+2,	"Cap.In3"	},
		{ "Capture.In4",			CAP_BEGIN+3,	"Cap.In4"	},
		{ "Capture.In5",			CAP_BEGIN+4,	"Cap.In5"	},
		{ "Capture.In6",			CAP_BEGIN+5,	"Cap.In6"	},
		{ "Capture.In7",			CAP_BEGIN+6,	"Cap.In7"	},
		{ "Capture.In8",			CAP_BEGIN+7,	"Cap.In8"	},
		
		{ "Capture.A", 				CAP_END,		"Cap.A"		},
		{ "Capture.B",				CAP_END-1,		"Cap.B"		},
		{ "Capture.C", 				CAP_END-2,		"Cap.C"		},
		{ "Capture.D",				CAP_END-3,		"Cap.D"		},
		{ "Capture.E",				CAP_END-4,		"Cap.E"		},
		{ "Capture.F",				CAP_END-5,		"Cap.F"		},
		{ "Capture.G",				CAP_END-6,		"Cap.G"		},
		{ "Capture.H",				CAP_END-7,		"Cap.H"		},

		{ "Capture.DistFieldSrc",	CAP_END-16,		"Cap.DFSrc"	},
		{ "Capture.DistField",		CAP_END-17,		"Cap.DF"	},
		{ "Capture.FieldA",			CAP_END-18,		"Cap.FA"	},
		{ "Capture.FieldB",			CAP_END-19,		"Cap.FB"	},

--todo we should generate Monitor on the fly
		{ "KinA",					256+96,			"M1"		},
		{ "KinB",					256+96+1,		"M2"		},
		{ "KinC",					256+96+2,		"M3"		},
	}

	bind_tex:add_pre( "video",	function(i) return string.char(64+i) end,	96,			16 )
	bind_tex:add_pre( "fbo",	"F",										64,			24 )
	bind_tex:add_pre( "in",		"IN",										128,		32 )
	bind_tex:add_pre( "mire",	"Mi",										128 + 32,	16 )
	bind_tex:add_pre( "M",		"M",										256 + 32*3, 32 )


	--todo make sur this mapped well always
	bind_tex:add_name_bind( "Util", "Noise", 63 )
	bind_tex:add_name_bind( "Util", "UVMap", 33 )

	bind_tex:add_table( "dif", 		dif )
	bind_tex:add_table( "pass", 	pass )
	bind_tex:add_table( "capture", 	cap )

	-- local function add_to_mon_list( start, )

	-- local tab = {}
	-- for i = 1,16*5 do tab[i]="No" end

	bind_tex:compute()

	TEXS.__bind_tex = bind_tex
	TEXS.BIND_HACK_BEGIN = 256
	TEXS.BIND_HACK_END = TEXS.BIND_HACK_BEGIN + 32 - 1
	TEXS.BIND_PB = 127

--	table.print( bind_tex.__name_to_info, "name_to_info", 2 )
--	aaa.box_debug( "bind_tex" )
end

function TEXS:get_tex_named()	--todotex rename and restructurefor clarity
	return self.__bind_tex
--	local tab = self.__bind_tex
--	return tab and tab or MEU:__init_tex_named()
end
function TEXS:get_bind_by_name( name )		return self:get_tex_named():get_bind_by_name( name )	end
function TEXS:get_name_from_bind( bind )	return self:get_tex_named():get_name_from_bind( bind )	end
function TEXS:get_name_or_bind( bind )		return self:get_tex_named():get_name_or_bind( bind )	end

function TEXS:set_bank_names( bu )
	bu:set_item_text( 3, "F", "Vid", "IN", "Mi", "Fbo"	)
	bu:set_item_text( 9, "[ GaBuZo", "9",  "10",  "11 ]", "[ User", "13", "14", "15 ]" )
	bu:set_item_text( 17, "[ VIDEO", "17", "18", "19 ]" )
	bu:set_item_text( 33, "[ IMAGE", "33", "34", "35", "36 . . ." )
end

function TEXS:__adjust_bind_bank_selector( bu, nb_u, line_nb )
	--	table.print( bu," BU_TEX_SEL;__adjust_bind_bank_selector", 2 )
	local nb = nb_u*line_nb
	bu:set_nb_min_0( nb_u, line_nb )
	for i=1,nb,4 do	bu:set_item_text( i, i-1 ) end
	for i=nb_u,nb,nb_u do bu:set_item_text( i, i-1 ) end
	bu:set_item_text( nb, nb-1 )
end
function TEXS:adjust_bank_selector( bu, nb_u, line_nb )
	--	table.print( bu, "TEXS;adjust_bank_selector()", 2 )
	self:__adjust_bind_bank_selector( bu, nb_u, line_nb )
	self:set_bank_names( bu )
end
function TEXS:adjust_bind_selector( bu, nb_u, line_nb )
	--	table.print( bu, "TEXS:adjust_bind_selector()", 2 )
	self:__adjust_bind_bank_selector( bu, nb_u, line_nb )
end

TEXS.__t_bind_text = {	"A","B","C","D",			"E","F","G","H",			"I","J","K","L", 				"Dif", "Pre", "DifR", "Viz",
						"F1","2","3","4",			"F5","6","7","8",			"F9","10","11","12", 			"F13","14","15","16",
						"In1","2","3","4",			"In5","6","7","8",			"In9","10","11","12",			"In13","14","15","16",
						"Mi1","2","3","4",			"Mi5","6","7","8",			"Mi9","10","11","12",			"Mi13","14","15","16",
						"Albe", "Nor", "Spec", "Emis",
													"ZBuf", "Light", "Caustic", "Fog",
																				"Dof", "Transparent", "GodRays", "ToneMap",
																												"Fxaa",nil,nil,"UVMap"
						}
TEXS.__t_bind_symbo = {	"A","B","C","D",			"E","F","G","H",			"I","J","K","L",				"Dif", "Pre", "DifR", "Viz",
						"F1","f2","f3","f4",		"F5","f6","f7","f8",		"F9","F10","F11","F12", 		"F13","F14","F15","F16",
						"In1","In2","In3","In4",	"In5","In6","In7","In8",	"In9","In10","In11","In12",		"In13","In14","In15","In16",
						"Mi1","Mi2","Mi3","Mi4",	"Mi5","Mi6","Mi7","Mi8",	"Mi9","Mi10","Mi11","Mi12",		"Mi13","Mi14","Mi15","Mi16",
						"Albedo", "Normal", "Specular", "Emissive",	
													"ZBuf", "Light", "Caustic", "Fog",
																				"Dof", "Transparent", "GodRays", "ToneMap",
																												"Fxaa",nil,nil,"UVMap"
						}
TEXS.__t_bind_order = {	1,2,3,4,		17,18,19,20,	33,34,35,36,	49,50,51,52,
						5,6,7,8,		21,22,23,24,	37,38,39,40,	53,54,55,56,
						9,10,11,12,		25,26,27,28,	41,42,43,44,	57,58,59,60,
						13,14,15,16,	29,30,31,32,	45,46,47,48,	61,62,63,64,
						65,66,67,68,	69,70,71,72,	73,74,75,76,	77	
						}

TEXS._t_dst_bind_text = {	"A","b","c","d",			"E","f","g","h",			"I","j","k","l",
							"F1","2","3","4",			"F5","6","7","8",			"F9","10","11","12",
							"In1","2","3","4",			"In5","6","7","8",			"In9","10","11","12",
						}
TEXS._t_dst_bind_symbo = {	"a","b","c","d",			"e","f","g","h",			"I","j","k","l",
							"F1","f2","f3","f4",		"F5","f6","f7","f8",		"F9","f10","f11","f12",
							"In1","In2","In3","In4",	"In5","In6","In7","In8",	"In9","In10","In11","In12",
						}
						
function TEXS:define_src_menu( bu, bind_text, bind_symbo, bind_order )
	bind_text = bind_text or self.__t_bind_text
	bind_symbo = bind_symbo or self.__t_bind_symbo
	bind_order = bind_order or self.__t_bind_order
	
	local nb_u = 16
	bu:set_fxy_max_for_menu( .3, 1, .417 )	--todo refine the max stuff just a hack now
	bu:set_menu( bind_text, bu:get_name() ):set_nb_min_0( nb_u, 5 )
	bu:set_value(80) --todo better

	local bu_sel = bu:get_selector()
	local nb = #bind_text
	for i=1,nb do
		local i_src = bind_order and bind_order[i] or i
		bu_sel:set_item_text( i, bind_text[i_src]  )
		bu_sel:set_item_symbo( i, bind_symbo[i_src] )
		--self:print( name.." - "..self:get_tex_named():get_bind_by_name( name ) )
		local name = bind_symbo[i_src]
		if name then		
			bu_sel:set_item_data( i, self:get_bind_by_name(name) )
		end
	end
	bu_sel.draw = function(self)
					SELECTOR.draw(self)
					aaa.draw_line( -.25,-.3, -.25, .5 )
					aaa.draw_line( 0   ,-.3, 0   , .5 )
					aaa.draw_line(  .25,-.3,  .25, .5 )
					aaa.draw_line( -.5 ,-.3,  .5 ,-.3 )
					aaa.draw_line( -.5 ,-.1, -.25,-.1 )
				end
	return bu
end

function TEXS:load_texture( bind )
	--aaa.print_fn()
	local str = "load textures at index "..bind
	local filter = "Reconnus\0*.bmp;*.gif;*.jpg;*.sgi;*.rgb;*.tga;*.tif;*.tiff;*.png;*.yuv;*.422;*.exr;*.dds\0".."Bmp\0*.bmp\0".."Gif\0*.gif\0".."Jpeg\0*.jpg\0".."Tga\0*.tga\0".."Tiff\0*.tif;*.tiff\0".."Png\0*.png\0".."Exr\0*.exr\0".."DDS\0*.dds\0".."Tous\0*.*\0".."\0"
	local filenames = aaa.file.do_dialog_open( str, filter, true )
	--table.print( filenames, "filenames" )
	for i,fname in IPAIRS(filenames) do
		self:print( i.." load "..fname.." at "..bind )
		aaa.img.read( bind, fname )
		bind = bind + 1
	end
end
