CLASS.DECLARE( "AAACAM" )

function AAACAM:create( name, obj_cam )
	AAACAM:set_class_status_doc(	CLASS.STATUS.CORE,
									"Encapsulate accesses to the C camera object" )

	local self = AAACAM:create_instance_with_obj( name, obj_cam )
	self.id = name	--	check if we need this

	local ref = self.ref
	if ref.obj then	--to deal with init process
		--todo add param.get_ref_xy, param.get_ref_xyz
		ref.ortho_size	=	param.get_ref( ref.obj, "ortho_size" )
		ref.ortho_sx	=	param.get_ref( ref.obj, "ortho_size_x" )
		ref.ortho_sy	=	param.get_ref( ref.obj, "ortho_size_y" )
		ref.b_viewport	=	param.get_ref( ref.obj, "sub_viewport" )
		ref.viewport_l	=	param.get_ref( ref.obj, "sub_viewport_left" )
		ref.viewport_r	=	param.get_ref( ref.obj, "sub_viewport_right" )
		ref.viewport_t	=	param.get_ref( ref.obj, "sub_viewport_top" )
		ref.viewport_b	=	param.get_ref( ref.obj, "sub_viewport_bottom" )
		ref.cen_x		=	param.get_ref( ref.obj, "center_x" )
		ref.cen_y		=	param.get_ref( ref.obj, "center_y" )
		ref.cen_z		=	param.get_ref( ref.obj, "center_z" )
		ref.tra_x		=	param.get_ref( ref.obj, "tra_x" )
		ref.tra_y		=	param.get_ref( ref.obj, "tra_y" )
		ref.tra_z		=	param.get_ref( ref.obj, "tra_z" )
		ref.focal		=	param.get_ref( ref.obj, "focal" )
		ref.frustum_x	=	param.get_ref( ref.obj, "frustum_offset_x" )
		ref.frustum_y	=	param.get_ref( ref.obj, "frustum_offset_y" )
	else
		self:box_error( "Can't found the AAASeed camera with this name" )
	end
	self:update()
	return self
end

function AAACAM:set_ortho_size( s )			param.set( self.ref.ortho_size, s )		end
function AAACAM:get_ortho_size()			return param.get( self.ref.ortho_size )	end
function AAACAM:get_ortho_max_size()		return self.ortho_sx, self.ortho_sy		end
function AAACAM:get_ratio()					return self.ratio end

function AAACAM:get_viewport()
	local ref = self.ref
	--self:print( param.get( ref.b_viewport ) )
	if param.get( ref.b_viewport ) > .5 then
		return param.get( ref.viewport_l ), param.get( ref.viewport_b ), param.get( ref.viewport_r ), param.get( ref.viewport_t )
	else
		return 0., 0., 1., 1.
	end
end

function AAACAM.set_viewport_from_ref( cam_ref, b_on, l, b, r, t )
	if cam_ref==0 then
		aaa.debug.print_traceback( "cam_ref is 0" )
		aaa.box_error( "AAACAM.set_viewport_from_ref() cam_ref is 0, GP rendering off probably (F3)" )
	else
		param.set( cam_ref, "sub_viewport", b_on )
		if l and l < r and b < t then --this is gl convention
			aaa.print( "viewport "..l.." "..b.." "..r.." "..t )
			param.set(	cam_ref,	"sub_viewport_left",	l	)
			param.set(	cam_ref,	"sub_viewport_bottom",	b	)
			param.set(	cam_ref,	"sub_viewport_right",	r	)
			param.set(	cam_ref,	"sub_viewport_top",		t	)
		end
	end
end
function AAACAM:set_viewport( b_on, l, b, r, t )
	local ref = self.ref
	param.set( ref.b_viewport, b_on )
	if b_on then
		param.set( ref.viewport_l, l )
		param.set( ref.viewport_b, b )
		param.set( ref.viewport_r, r )
		param.set( ref.viewport_t, t )
	end
end

function AAACAM:set_frustum( fx, fy )
	local ref = self.ref
	param.set( ref.frustum_x, fx )
	param.set( ref.frustum_y, fy )
end
function AAACAM:set_tra_z( z )	param.set( self.ref.tra_z, z )		end
function AAACAM:set_focal( f )	param.set( self.ref.focal, f )		end
function AAACAM:get_tra_xyz( z )
	local ref = self.ref
	return param.get( ref.tra_x ), param.get( ref.tra_y ), param.get( ref.tra_z )
end
function AAACAM:get_cen_xyz( z )
	local ref = self.ref
	return param.get( ref.cen_x ), param.get( ref.cen_y ), param.get( ref.cen_z )
end

function AAACAM:get_blobs_offset_scale()
	local sx, sy = self:get_ortho_max_size()
	--self:print( sx.." "..sy )
	--local l, b, r, t = self:get_viewport()
	--return -(r-l)*.5, -(t-b)*.5, (r-l)*8., (t-b)*sy*8./sx
	return -.5, -.5, sx, sy
end

function AAACAM:update()
	local ref = self.ref
	if ref.obj then
		aaa.obj.update_then_draw( ref.obj )
		self.ortho_sx = param.get( ref.ortho_sx )
		self.ortho_sy = param.get( ref.ortho_sy )
		self.ratio = self.ortho_sx / self.ortho_sy
	end
end

